namespace TitlepackUpdater{

	CGameUILayer@ loadingLayer;
	string scriptsRepositoryName = "https://raw.githubusercontent.com/ArkadySK/TMNextPlatform";
	string dataRepositoryName = "https://raw.githubusercontent.com/ArkadySK/TMNextPlatformTitlepack";
	string branch = "main";
	string userFolderPath;
	string ScriptVersion;

	bool IsUpToDate() {
		userFolderPath = IO::FromUserGameFolder("").Replace("\\", "/");
		
		auto@ netScriptVersion = Net::HttpGet(dataRepositoryName + "/" + branch + "/title_version.json");
		while (!netScriptVersion.Finished()) {
			yield();
		};
		ScriptVersion = netScriptVersion.String();
		string localVersionJSONPath = IO::FromDataFolder("Plugins/title_version.json"); 
		localVersionJSONPath = localVersionJSONPath.Replace("\\", "/");

		string localVersion = Json::Write(Json::FromFile(localVersionJSONPath)).Replace("\"", "");
		ScriptVersion = ScriptVersion.Replace("\"", "");
		localVersion = localVersion.Replace("\\", "");

		print("currentVersionJSONFile: " + localVersionJSONPath);
		print("Newest version: " + ScriptVersion);
		print("Version on your drive: " + localVersion);
		if(!HasMusic())
			return false;
		return ScriptVersion == localVersion;
	}

	bool HasMusic()	{
		string menuMusicFilePath = IO::FromUserGameFolder("Media\\Musics\\TMNextPlatform\\BlurredMemories.ogg");
    	return IO::FileExists(menuMusicFilePath);
	}

	void Update(CGameUILayer@ layer)
	{
		@loadingLayer = layer;
		loadingLayer.ManialinkPage = LoadscreenText("Please wait...");
		
		DownloadScripts();
		try	{
			DownloadCoreFiles();
			sleep(1000);
			GetMenuCustom().Dialog_Message("Successfully updated to version: " + ScriptVersion + "\nPlease restart your game to apply changes.");
		}
		catch	{
			GetMenuCustom().Dialog_Message("Download failed. Please report this issue.");
			return;
		}

		string currentVersionJSON = IO::FromDataFolder("Plugins/title_version.json");
		currentVersionJSON = currentVersionJSON.Replace("\\", "/");
		Json::ToFile(currentVersionJSON, ScriptVersion);
		print("UPDATED: saving current version file, ScriptVersion:" + ScriptVersion);

		UI::ShowNotification("\\$0d0" + Icons::Kenney::Flag + "Platform titlepack has been updated.", "\\$fffRunning version " + ScriptVersion + ".", 10000);	
	}


	void DownloadCoreFiles() {
		UI::ShowNotification("\\$0cf" + Icons::Download + "Redownloading Platform titlepack...", "\\$fffDowloading all files...", 10000);
		
		// Media/Images/Icon
		auto@ netMediaImagesTMNextPlatformIcon_discord = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/Icon/discord.png");
		auto@ netMediaImagesTMNextPlatformIcon_Github = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/Icon/Github.png");
		
		// Media/Images
		auto@ netMediaImagesTMNextPlatform_Loadscreen = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/Loadscreen.jpg");
		auto@ netMediaImagesTMNextPlatformBackground_MenuBg = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/Background/MenuBg.webm");
		auto@ netMediaImagesTMNextPlatform_PlatformBackground = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/PlatformBackground.jpg");
		
		// Media/Sounds
		auto@ netMediaSoundsTMNextPlatform_Empty = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Sounds/TMNextPlatform/empty.wav");

		loadingLayer.ManialinkPage = LoadscreenText("Updating data...\nDownloading media...");
		while (!netMediaImagesTMNextPlatformIcon_discord.Finished() || 
				!netMediaImagesTMNextPlatformIcon_Github.Finished() || 

				!netMediaImagesTMNextPlatform_Loadscreen.Finished() || 
				!netMediaImagesTMNextPlatformBackground_MenuBg.Finished() || 
				!netMediaImagesTMNextPlatform_PlatformBackground.Finished() ||  
				
				!netMediaSoundsTMNextPlatform_Empty.Finished()){
			yield();
		}

		// Media/Musics
		auto@ netMediaMusicsTMNextPlatform_BlurredMemories = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/TMNextPlatform/BlurredMemories.ogg");
		auto@ netMediaMusicsTMNextPlatform_DeveloperNightClub = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/TMNextPlatform/DeveloperNightClub.ogg");
		auto@ netMediaMusicsTMNextPlatform_Heartbeat = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/TMNextPlatform/Heartbeat.ogg");
		
		auto@ netMediaMusicsCanyonPlatformRace_BuffaloSkull = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/CanyonPlatform/Race/Buffalo%20Skull.ogg");
		auto@ netMediaMusicsCanyonPlatformRace_FasterThanLight = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/CanyonPlatform/Race/Faster%20Than%20Light.ogg");
		auto@ netMediaMusicsCanyonPlatformRace_SandStorms = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/CanyonPlatform/Race/Sand%20Storms.ogg");
		auto@ netMediaMusicsCanyonPlatformRace_TheLostOasis = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/CanyonPlatform/Race/The%20Lost%20Oasis.ogg");
		
		loadingLayer.ManialinkPage = LoadscreenText("Updating data...\nDownloading musics...");
		while (!netMediaMusicsTMNextPlatform_BlurredMemories.Finished() || 
				!netMediaMusicsTMNextPlatform_DeveloperNightClub.Finished() || 
				!netMediaMusicsTMNextPlatform_Heartbeat.Finished() ||
				
				!netMediaMusicsCanyonPlatformRace_BuffaloSkull.Finished() || 
				!netMediaMusicsCanyonPlatformRace_FasterThanLight.Finished() || 
				!netMediaMusicsCanyonPlatformRace_SandStorms.Finished() || 
				!netMediaMusicsCanyonPlatformRace_TheLostOasis.Finished()){
			yield();
		}
    
		// Maps
		auto@ netMapsTMNextPlatform0_PlatformDemo01 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo01.Map.Gbx");
		auto@ netMapsTMNextPlatform0_PlatformDemo02 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo02.Map.Gbx");
		auto@ netMapsTMNextPlatform0_PlatformDemo03 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo03.Map.Gbx");
		auto@ netMapsTMNextPlatform0_PlatformDemo04 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo04.Map.Gbx");
		auto@ netMapsTMNextPlatform0_PlatformDemo05 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo05.Map.Gbx");
		
		loadingLayer.ManialinkPage = LoadscreenText("Updating data...\nDownloading campaign maps...");
		while (	!netMapsTMNextPlatform0_PlatformDemo01.Finished() || 
				!netMapsTMNextPlatform0_PlatformDemo02.Finished() || 
				!netMapsTMNextPlatform0_PlatformDemo03.Finished() || 
				!netMapsTMNextPlatform0_PlatformDemo04.Finished() || 
				!netMapsTMNextPlatform0_PlatformDemo05.Finished() ){
			yield();
		}

		
		print("IO: creating folders: Media, Maps, Sounds");
		IO::CreateFolder(userFolderPath + "/Media");
		IO::CreateFolder(userFolderPath + "/Media/Images");
		IO::CreateFolder(userFolderPath + "/Media/Images/TMNextPlatform");
		IO::CreateFolder(userFolderPath + "/Media/Images/TMNextPlatform/Icon");
		IO::CreateFolder(userFolderPath + "/Media/Images/TMNextPlatform/Background");
		IO::CreateFolder(userFolderPath + "/Media/Sounds");
		IO::CreateFolder(userFolderPath + "/Media/Sounds/TMNextPlatform");
		IO::CreateFolder(userFolderPath + "/Media/Musics");
		IO::CreateFolder(userFolderPath + "/Media/Musics/TMNextPlatform");
		IO::CreateFolder(userFolderPath + "/Media/Musics/CanyonPlatform");
		IO::CreateFolder(userFolderPath + "/Media/Musics/CanyonPlatform/Race");
		IO::CreateFolder(userFolderPath + "/Maps");
		IO::CreateFolder(userFolderPath + "/Maps/");
		IO::CreateFolder(userFolderPath + "/Maps/TMNextPlatform/0");

	
		print("IO: writing main files...");
		IO::File fileMediaImagesTMNextPlatformIcon_discord(userFolderPath + "/Media/Images/TMNextPlatform/Icon/discord.png", IO::FileMode::Write);
			fileMediaImagesTMNextPlatformIcon_discord.Write(netMediaImagesTMNextPlatformIcon_discord.String());
		IO::File fileMediaImagesTMNextPlatformIcon_Github(userFolderPath + "/Media/Images/TMNextPlatform/Icon/Github.png", IO::FileMode::Write);
			fileMediaImagesTMNextPlatformIcon_Github.Write(netMediaImagesTMNextPlatformIcon_Github.String());

		IO::File fileMediaImagesTMNextPlatform_Loadscreen(userFolderPath + "/Media/Images/TMNextPlatform/Loadscreen.jpg", IO::FileMode::Write);
			fileMediaImagesTMNextPlatform_Loadscreen.Write(netMediaImagesTMNextPlatform_Loadscreen.String());		
		try{
			IO::File fileMediaImagesTMNextPlatformBackground_MenuBg(userFolderPath + "/Media/Images/TMNextPlatform/Background/MenuBg.webm", IO::FileMode::Write);
				fileMediaImagesTMNextPlatformBackground_MenuBg.Write(netMediaImagesTMNextPlatformBackground_MenuBg.String());
		}
		catch{
			print("error saving Background/MenuBg.webm file.");
		}
		IO::File fileMediaImagesTMNextPlatform_PlatformBackground(userFolderPath + "/Media/Images/TMNextPlatform/PlatformBackground.jpg", IO::FileMode::Write);
			fileMediaImagesTMNextPlatform_PlatformBackground.Write(netMediaImagesTMNextPlatform_PlatformBackground.String());
		
		IO::File fileMediaMusicsTMNextPlatform_BlurredMemories(userFolderPath + "/Media/Musics/TMNextPlatform/BlurredMemories.ogg", IO::FileMode::Write);
			fileMediaMusicsTMNextPlatform_BlurredMemories.Write(netMediaMusicsTMNextPlatform_BlurredMemories.String());
		IO::File fileMediaMusicsTMNextPlatform_DeveloperNightClub(userFolderPath + "/Media/Musics/TMNextPlatform/DeveloperNightClub.ogg", IO::FileMode::Write);
			fileMediaMusicsTMNextPlatform_DeveloperNightClub.Write(netMediaMusicsTMNextPlatform_DeveloperNightClub.String());
		IO::File fileMediaMusicsTMNextPlatform_Heartbeat(userFolderPath + "/Media/Musics/TMNextPlatform/Heartbeat.ogg", IO::FileMode::Write);
			fileMediaMusicsTMNextPlatform_Heartbeat.Write(netMediaMusicsTMNextPlatform_Heartbeat.String());

		IO::File fileMediaMusicsCanyonPlatformRace_BuffaloSkull(userFolderPath + "/Media/Musics/CanyonPlatform/Race/Buffalo Skull.ogg", IO::FileMode::Write);
			fileMediaMusicsCanyonPlatformRace_BuffaloSkull.Write(netMediaMusicsCanyonPlatformRace_BuffaloSkull.String());
		IO::File fileMediaMusicsCanyonPlatformRace_FasterThanLight(userFolderPath + "/Media/Musics/CanyonPlatform/Race/Faster Than Light.ogg", IO::FileMode::Write);
			fileMediaMusicsCanyonPlatformRace_FasterThanLight.Write(netMediaMusicsCanyonPlatformRace_FasterThanLight.String());
		IO::File fileMediaMusicsCanyonPlatformRace_SandStorms(userFolderPath + "/Media/Musics/CanyonPlatform/Race/Sand Storms.ogg", IO::FileMode::Write);
			fileMediaMusicsCanyonPlatformRace_SandStorms.Write(netMediaMusicsCanyonPlatformRace_SandStorms.String());
		IO::File fileMediaMusicsCanyonPlatformRace_TheLostOasis(userFolderPath + "/Media/Musics/CanyonPlatform/Race/The Lost Oasis.ogg", IO::FileMode::Write);
			fileMediaMusicsCanyonPlatformRace_TheLostOasis.Write(netMediaMusicsCanyonPlatformRace_TheLostOasis.String());
		
		IO::File fileMediaSoundsTMNextPlatform_Empty(userFolderPath + "/Media/Sounds/TMNextPlatform/empty.wav", IO::FileMode::Write);
			fileMediaSoundsTMNextPlatform_Empty.Write(netMediaSoundsTMNextPlatform_Empty.String());
		
		IO::File fileMapsTMNextPlatform0_PlatformDemo01(userFolderPath + "/Maps/TMNextPlatform/0/PlatformDemo01.Map.Gbx", IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo01.Write(netMapsTMNextPlatform0_PlatformDemo01.String());
		IO::File fileMapsTMNextPlatform0_PlatformDemo02(userFolderPath + "/Maps/TMNextPlatform/0/PlatformDemo02.Map.Gbx", IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo02.Write(netMapsTMNextPlatform0_PlatformDemo02.String());
		IO::File fileMapsTMNextPlatform0_PlatformDemo03(userFolderPath + "/Maps/TMNextPlatform/0/PlatformDemo03.Map.Gbx", IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo03.Write(netMapsTMNextPlatform0_PlatformDemo03.String());
		IO::File fileMapsTMNextPlatform0_PlatformDemo04(userFolderPath + "/Maps/TMNextPlatform/0/PlatformDemo04.Map.Gbx", IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo04.Write(netMapsTMNextPlatform0_PlatformDemo04.String());
		IO::File fileMapsTMNextPlatform0_PlatformDemo05(userFolderPath + "/Maps/TMNextPlatform/0/PlatformDemo05.Map.Gbx", IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo05.Write(netMapsTMNextPlatform0_PlatformDemo05.String());

	}

	void DownloadScripts() {
		UI::ShowNotification("\\$0cf" + Icons::Download + "Updating Platform titlepack...", "\\$fffDowloading update...", 3000);

		// Scripts
		auto@ netLibs_ModeManialinks = Net::HttpGet(scriptsRepositoryName + "/" + branch + "/Libs/TrackMania/ModeManialinks.Script.txt");
		auto@ netMapTypes_Platform = Net::HttpGet(scriptsRepositoryName + "/" + branch + "/MapTypes/Trackmania/Platform.Script.txt");
		auto@ netModes_LegacyMode = Net::HttpGet(scriptsRepositoryName + "/" + branch + "/Modes/TrackMania/LegacyMode.Script.txt");
		auto@ netModes_PlatformRounds = Net::HttpGet(scriptsRepositoryName + "/" + branch + "/Modes/TrackMania/PlatformRounds.Script.txt");
		
		loadingLayer.ManialinkPage = LoadscreenText("Downloading scripts and menus...");
		while (!netLibs_ModeManialinks.Finished() || 
				!netMapTypes_Platform.Finished() || 
				!netModes_LegacyMode.Finished() || 
				!netModes_PlatformRounds.Finished()){
			yield();
		}

		
		print("IO: creating folders: Scripts");
		IO::CreateFolder(userFolderPath + "/Scripts");
		IO::CreateFolder(userFolderPath + "/Scripts/Libs");
		IO::CreateFolder(userFolderPath + "/Scripts/Libs/Trackmania");
		IO::CreateFolder(userFolderPath + "/Scripts/Modes");
		IO::CreateFolder(userFolderPath + "/Scripts/Modes/Trackmania");
		IO::CreateFolder(userFolderPath + "/Scripts/MapTypes");
		IO::CreateFolder(userFolderPath + "/Scripts/MapTypes/Trackmania");

		print("userFolderPath:" + userFolderPath);
		
		print("writing scripts...");
		IO::File fileLibsModeManialinks(userFolderPath + "/Scripts/Libs/TrackMania/ModeManialinks.Script.txt", IO::FileMode::Write);
		fileLibsModeManialinks.Write(netLibs_ModeManialinks.String());
		IO::File fileMapTypesPlatform(userFolderPath + "/Scripts/MapTypes/Trackmania/Platform.Script.txt", IO::FileMode::Write);
		fileMapTypesPlatform.Write(netMapTypes_Platform.String());
		IO::File fileModesLegacyMode(userFolderPath + "/Scripts/Modes/TrackMania/LegacyMode.Script.txt", IO::FileMode::Write);
		fileModesLegacyMode.Write(netModes_LegacyMode.String());
		IO::File fileModesPlatformRounds(userFolderPath + "/Scripts/Modes/TrackMania/PlatformRounds.Script.txt", IO::FileMode::Write);
		fileModesPlatformRounds.Write(netModes_PlatformRounds.String());
	}
}
